@@ -99,6 +99,9 @@ typedef enum {
     XML_SCHEMA_TYPE_LIST,
     XML_SCHEMA_TYPE_UNION,
     XML_SCHEMA_TYPE_ANY_ATTRIBUTE,
+    XML_SCHEMA_TYPE_IDC_UNIQUE,
+    XML_SCHEMA_TYPE_IDC_KEY,
+    XML_SCHEMA_TYPE_IDC_KEYREF,
     XML_SCHEMA_FACET_MININCLUSIVE = 1000,
     XML_SCHEMA_FACET_MINEXCLUSIVE,
     XML_SCHEMA_FACET_MAXINCLUSIVE,
@@ -204,7 +207,7 @@ struct _xmlSchemaAnnot {
  */
 #define XML_SCHEMAS_ATTR_USE_OPTIONAL 2
 /**
- * XML_SCHEMAS_ATTR_GLOABAL:
+ * XML_SCHEMAS_ATTR_GLOBAL:
  *
  * allow elements in no namespace
  */
@@ -350,6 +353,7 @@ struct _xmlSchemaAttributeGroup {
     xmlSchemaWildcardPtr attributeWildcard;
     const xmlChar *refPrefix;
     xmlSchemaAttributeGroupPtr refItem; /* The referenced attribute group */
+    const xmlChar *targetNamespace;
 };
 
 /**
@@ -455,19 +459,19 @@ struct _xmlSchemaFacetLink {
  */
 #define XML_SCHEMAS_TYPE_FINAL_UNION    1 << 12
 /**
- * XML_SCHEMAS_TYPE_FINAL_UNION:
+ * XML_SCHEMAS_TYPE_FINAL_DEFAULT:
  *
- * the simpleType has a final of "union".
+ * the simpleType has a final of "default".
  */
 #define XML_SCHEMAS_TYPE_FINAL_DEFAULT    1 << 13
 /**
- * XML_SCHEMAS_TYPE_FINAL_UNION:
+ * XML_SCHEMAS_TYPE_BUILTIN_PRIMITIVE:
  *
- * the simpleType has a final of "union".
+ * Marks the item as a builtin primitive.
  */
 #define XML_SCHEMAS_TYPE_BUILTIN_PRIMITIVE    1 << 14
 /**
- * XML_SCHEMAS_TYPE_MARKED
+ * XML_SCHEMAS_TYPE_MARKED:
  *
  * Marks the item as marked; used for circular checks.
  */
@@ -486,7 +490,7 @@ struct _xmlSchemaFacetLink {
  */
 #define XML_SCHEMAS_TYPE_BLOCK_EXTENSION    1 << 18
 /**
- * XML_SCHEMAS_TYPE_FINAL_RESTRICTION:
+ * XML_SCHEMAS_TYPE_BLOCK_RESTRICTION:
  *
  * the complexType has a 'block' of "restriction".
  */
@@ -497,6 +501,12 @@ struct _xmlSchemaFacetLink {
  * the simple/complexType is abstract.
  */
 #define XML_SCHEMAS_TYPE_ABSTRACT    1 << 20
+/**
+ * XML_SCHEMAS_TYPE_FACETSNEEDVALUE:
+ *
+ * indicates if the facets need a computed value
+ */
+#define XML_SCHEMAS_TYPE_FACETSNEEDVALUE    1 << 21
 
 /**
  * _xmlSchemaType:
@@ -533,6 +543,7 @@ struct _xmlSchemaType {
     const xmlChar *refPrefix;
     xmlSchemaTypePtr contentTypeDef;
     xmlRegexpPtr contModel;
+    const xmlChar *targetNamespace;
 };
 
 /*
@@ -593,14 +604,14 @@ struct _xmlSchemaType {
  */
 #define XML_SCHEMAS_ELEM_NSDEFAULT        1 << 7
 /**
- * XML_SCHEMAS_ELEM_INTERNAL_RESOLVED
+ * XML_SCHEMAS_ELEM_INTERNAL_RESOLVED:
  *
  * this is set when "type", "ref", "substitutionGroup"
  * references have been resolved.
  */
 #define XML_SCHEMAS_ELEM_INTERNAL_RESOLVED        1 << 8
  /**
- * XML_SCHEMAS_ELEM_CIRCULAR
+ * XML_SCHEMAS_ELEM_CIRCULAR:
  *
  * a helper flag for the search of circular references.
  */
@@ -636,13 +647,13 @@ struct _xmlSchemaType {
  */
 #define XML_SCHEMAS_ELEM_FINAL_ABSENT        1 << 14
 /**
- * XML_SCHEMAS_ELEM_BLOCK_EXTENSION:
+ * XML_SCHEMAS_ELEM_FINAL_EXTENSION:
  *
  * substitution group exclusions: "extension"
  */
 #define XML_SCHEMAS_ELEM_FINAL_EXTENSION        1 << 15
 /**
- * XML_SCHEMAS_ELEM_BLOCK_RESTRICTION:
+ * XML_SCHEMAS_ELEM_FINAL_RESTRICTION:
  *
  * substitution group exclusions: "restriction"
  */
@@ -678,6 +689,7 @@ struct _xmlSchemaElement {
     xmlSchemaContentType contentType;
     const xmlChar *refPrefix;
     xmlSchemaValPtr defVal;
+    void *idcs;
 };
 
 /*
@@ -730,6 +742,7 @@ struct _xmlSchemaNotation {
     const xmlChar *name;
     xmlSchemaAnnotPtr annot;
     const xmlChar *identifier;
+    const xmlChar *targetNamespace;
 };
 
 /**
@@ -821,6 +834,7 @@ struct _xmlSchema {
     void *includes;     /* the includes, this is opaque for now */
     int preserve;        /* whether to free the document */
     int counter; /* used to give ononymous components unique names */
+    xmlHashTablePtr idcDef;
 };
 
 XMLPUBFUN void XMLCALL         xmlSchemaFreeType        (xmlSchemaTypePtr type);
