@@ -86,6 +86,19 @@ XMLPUBFUN int XMLCALL
 					 const xmlChar *value2,
 					 void *data);
 
+XMLPUBFUN int XMLCALL
+		    xmlRegExecNextValues(xmlRegExecCtxtPtr exec,
+		    			 int *nbval,
+		    			 int *nbneg,
+					 xmlChar **values,
+					 int *terminal);
+XMLPUBFUN int XMLCALL
+		    xmlRegExecErrInfo	(xmlRegExecCtxtPtr exec,
+		    			 const xmlChar **string,
+					 int *nbval,
+		    			 int *nbneg,
+					 xmlChar **values,
+					 int *terminal);
 #ifdef __cplusplus
 }
 #endif 
